(* ******************************************************************** *)
(*
 * Project         : HOL-CSP - A Shallow Embedding of CSP in  Isabelle/HOL
 * Version         : 2.0
 *
 * Author          : Burkhart Wolff
 *                   (Based on HOL-CSP 1.0 by Haykal Tej and Burkhart Wolff)
 *
 * This file       : The sequence operator definitions
 *
 * Copyright (c) 2009 Universit Paris-Sud, France
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id: ROOT.ML 8591 2009-09-26 09:58:14Z brucker $ *)

header{* The Sequence Operator *}

theory  Seq
imports Process 
        (* Legacy *)
begin 

constdefs  seq :: "['a process,'a process] => 'a process"  (infixl  "`;`" 24)  
              "P `;` Q \<equiv> Abs_Process
                            ({(t, X). (t, X \<union> {tick}) \<in> F P \<and> tickFree t} \<union>
                             {(t, X). \<exists>t1 t2. t = t1 @ t2 \<and> t1 @ [tick] \<in> T P \<and> (t2, X) \<in> F Q} \<union>
                             {(t, X). \<exists>t1 t2. t = t1 @ t2 \<and> t1 \<in> D P \<and> tickFree t1 \<and> front_tickFree t2} \<union>
                             {(t, X). \<exists>t1 t2. t = t1 @ t2 \<and> t1 @ [tick] \<in> T P \<and> t2 \<in> D Q},
                             {t1 @ t2 |t1 t2. t1 \<in> D P \<and> tickFree t1 \<and> front_tickFree t2} \<union>
                             {t1 @ t2 |t1 t2. t1 @ [tick] \<in> T P \<and> t2 \<in> D Q})"


axioms 
    F_seq   : "F(P `;` Q) =  {(t, X). (t, X \<union> {tick}) \<in> F P \<and> tickFree t} \<union>
                             {(t, X). \<exists>t1 t2. t = t1 @ t2 \<and> t1 @ [tick] \<in> T P \<and> (t2, X) \<in> F Q} \<union>
                             {(t, X). \<exists>t1 t2. t = t1 @ t2 \<and> t1 \<in> D P \<and> tickFree t1 \<and> front_tickFree t2} \<union>
                             {(t, X). \<exists>t1 t2. t = t1 @ t2 \<and> t1 @ [tick] \<in> T P \<and> t2 \<in> D Q}"

    D_seq   : "D(P `;` Q) =  {t1 @ t2 |t1 t2. t1 \<in> D P \<and> tickFree t1 \<and> front_tickFree t2} \<union>
                             {t1 @ t2 |t1 t2. t1 @ [tick] \<in> T P \<and> t2 \<in> D Q}"

    T_seq   : "T(P `;` Q) =  {t. \<exists> X. (t, X \<union> {tick}) \<in> F P \<and> tickFree t} \<union>    (* REALLY ??? *)
                             {t. \<exists> t1 t2. t = t1 @ t2 \<and> t1 @ [tick] \<in> T P \<and> t2 \<in> T Q} \<union>
                             {t1 @ t2 |t1 t2. t1 \<in> D P \<and> tickFree t1 \<and> front_tickFree t2} \<union>
                             {t1 @ t2 |t1 t2. t1 @ [tick] \<in> T P \<and> t2 \<in> D Q}"

    seq_cont: "\<lbrakk>cont f; cont g\<rbrakk> \<Longrightarrow> cont (\<lambda> x. f x `;` g x)"

end

(* imported from ProcessType \<dots> *)
(*
val prem = goal Trace.thy
"[|~ tickFree s ; front_tickFree s|] ==> ? t. s = t @ [tick]";
by(cut_facts_tac prem 1);
by(res_inst_tac [("x","rev(tl (rev s))")] exI 1);
by(asm_simp_tac(HOL_ss addsimps [NtF_ftF RS sym])1);
qed "NtF_ftF_ex";
*)
